from pathlib import Path
import logging

from machine_admin.package_manager import SystemPackage
from machine_admin.util import run_op

logger = logging.getLogger(__name__)


class DebPreseeder:

    def __init__(self, iso_path: Path, preseed_path: Path):
        self.iso_path = iso_path
        self.preseed_path = preseed_path

        self.udevil = SystemPackage("udevil")
        self.gzip = SystemPackage("gzip")
        self.mount_point = None

    def mount_iso(self):
        self.udevil.check_available()
        op = f"udevil mount {self.iso_path}"
        logger.info(f"Mounting iso | {op}")
        output = run_op(op, capture_output=True)
        self.mount_point = Path(output.split(" ")[3])

    def add_preseed_file(self):

        # Copy ISO content to writeable location
        op = f"cp -rT {self.mount_point} {self.workdir}/isofiles/"
        run_op(op)

        # Make path writeable
        install_arch_path = f"{self.workdir}/isofiles/install.amd"
        run_op(f"chmod +w -R {install_arch_path}")

        # Extract initrd
        self.gzip.check_available()
        initrd_path = f"{install_arch_path}/initrd"
        run_op(f"gunzip {initrd_path}.gz")

        # Add the preseed file
        op = f"echo {self.preseed_path} | cpio -H newc -o -A -F {initrd_path}"
        run_op(op)

        # Recompress the initrd
        run_op(f"gzip {initrd_path}")

        # Restore path permissions
        run_op(f"chmod -w -R {install_arch_path}")

    def regenerate_checksum(self):
        checksum_path = f"{self.workdir}/isofiles/md5sum.txt"
        run_op(f"chmod +w {checksum_path}")

    def regenerate_iso(self, workdir: Path):

        self.mount_iso()

        self.add_preseed_file()

        self.regenerate_checksum()
