import argparse
import logging
from pathlib import Path

from jgutils import music
from jgutils.serialization import write_model

logger = logging.getLogger(__name__)

def cli_music_read(args):
    collection = music.collection.read(args.collection.resolve())
    write_model(collection)

def cli_music_refresh(args):
    music.collection.refresh(args.collection.resolve(),
                             args.work_dir.resolve())

def main_cli():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(required=True)

    music_parser = subparsers.add_parser("music")
    music_subparsers = music_parser.add_subparsers(required=True)
   
    music_read_parser = music_subparsers.add_parser("read")
    music_read_parser.add_argument(
        "--collection",
        type=Path,
        default=Path(),
        help="Directory with input music files.",
    )
    music_read_parser.add_argument(
        "--output_path",
        type=Path,
        default=Path() / "music_collection.json",
        help="Path to save collection info json to.",
    )
    music_read_parser.set_defaults(func=cli_music_read)

    music_refresh_parser = music_subparsers.add_parser("refresh")
    music_refresh_parser.add_argument(
        "--collection",
        type=Path,
        default=Path(),
        help="Directory with the music collection.",
    )
    music_refresh_parser.add_argument(
        "--work_dir",
        type=Path,
        default=Path(),
        help="Directory for intermediate storage.",
    )
    music_refresh_parser.set_defaults(func=cli_music_refresh)

    logging.basicConfig()
    logging.getLogger().setLevel(logging.INFO)

    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main_cli()
