import argparse
import os
import logging
import shutil
from pathlib import Path
import subprocess
import uuid

from fabric import Connection

logger = logging.getLogger(__name__)


def sync(source_dir: Path, target_dir: Path, host: str):

    logger.info("Syncing files in %s to %s on % s", source_dir, target_dir, host)

    cmd = f"find {target_dir} -type f"

    if host:
        result = Connection(host).run(cmd, hide=True)
        target_files = result.stdout.splitlines()
    else:
        result = subprocess.run(cmd, shell=True, capture_output=True, text=True)
        target_files = result.stdout.splitlines()

    target_files = [Path(f).relative_to(target_dir) for f in target_files]
    source_files = []
    for dir_path, _, files in os.walk(source_dir):
        for each_file in files:
            source_files.append(
                Path(dir_path + "/" + each_file).relative_to(source_dir)
            )

    sync_files = []
    for source_file in source_files:
        if source_file not in target_files:
            sync_files.append(source_file)

    sync_dir = Path(os.getcwd()) / "sync"
    os.makedirs(sync_dir, exist_ok=True)
    for sync_file in sync_files:
        os.makedirs((sync_dir / sync_file).parent, exist_ok=True)
        shutil.copy(source_dir / sync_file, sync_dir / sync_file)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()

    parser.add_argument(
        "--source_dir",
        type=Path,
        default=Path(),
        help="Directory with source files to sync from.",
    )

    parser.add_argument(
        "--host", type=str, default="", help="Name of host system to sync with."
    )

    parser.add_argument(
        "--target_dir",
        type=Path,
        default=Path(),
        help="Directory with source files to sync to.",
    )

    args = parser.parse_args()

    logging.basicConfig()
    logging.getLogger().setLevel(logging.INFO)

    sync(args.source_dir.resolve(), args.target_dir.resolve(), args.host)
